package com.jsh.erp.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jsh.erp.constants.ExceptionConstants;
import com.jsh.erp.datasource.dto.AuditDTO;
import com.jsh.erp.datasource.dto.AuditQueryDTO;
import com.jsh.erp.datasource.dto.MaterialDTO;
import com.jsh.erp.datasource.dto.MaterialExtendDTO;
import com.jsh.erp.datasource.entities.*;
import com.jsh.erp.datasource.mappers.*;
import com.jsh.erp.datasource.pda.dto.PDADepotHeadDTO;
import com.jsh.erp.datasource.pda.vo.PDADepotHeadVO;
import com.jsh.erp.datasource.vo.AuditNodeVo;
import com.jsh.erp.datasource.vo.AuditVo;
import com.jsh.erp.exception.BusinessRunTimeException;
import com.jsh.erp.exception.ServiceException;
import com.jsh.erp.query.LambdaQueryWrapperX;
import com.jsh.erp.service.*;
import com.jsh.erp.utils.PageUtils;
import com.jsh.erp.utils.StringUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class AuditServiceImpl extends ServiceImpl<AuditMapper, Audit> implements AuditService {

    private final AuditMapper auditMapper;

    private final AuditNodeConfigMapper auditNodeConfigMapper;

    private final AuditNodeMapper auditNodeMapper;

    private final AuditProcessMapper auditProcessMapper;

    private final UserService userService;

    @Resource
    private MaterialService materialService;
    @Resource
    private MaterialBatchService materialBatchService;
    @Resource
    private DepotHeadService depotHeadService;
    @Resource
    private MaterialInputService materialInputService;
    @Resource
    private TaskStocktakingService taskStocktakingService;

    /**
     * 根据审核业务单号和类型生成审核实例及节点实例
     *
     * @param businessNumber 业务单号
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public boolean generateAuditByItemId(String businessNumber, String type) {
        try {
            //针对重新提交申请，获取失败审核
            Audit a = getOne(new LambdaQueryWrapperX<Audit>()
                    .eq(Audit::getBusinessNumber, businessNumber)
                    .eq(Audit::getAuditStatus, 2)
                    .eq(Audit::getDeleteFlag, false));
            if (a != null) {
                //删除审核
                update(new UpdateWrapper<Audit>().set("delete_flag", true).eq("id", a.getId()));
            }
            //获取业务id
            Long bid;
            if (type.equals("盘点任务")) {
                bid = taskStocktakingService.getOne(new LambdaQueryWrapperX<TaskStocktaking>().eq(TaskStocktaking::getNumber, businessNumber)).getId();
            } else if (type.equals("pda维护商品")) {
                bid = materialInputService.getOne(new LambdaQueryWrapperX<MaterialInput>().eq(MaterialInput::getNumber, businessNumber)).getId();
            } else {
                bid = depotHeadService.getDepotHead(businessNumber).getId();
            }
            //获取审核流程
            AuditProcess auditProcess = auditProcessMapper.selectOne(new LambdaQueryWrapperX<AuditProcess>()
                    .eq(AuditProcess::getType, type)
                    .eq(AuditProcess::getDeleteFlag, false));
            //判断是否有审核流程
            if (auditProcess != null) {
                //有审核流程，根据流程创建审核实力及第一个节点实例
                Audit audit = new Audit();
                audit.setProcessId(auditProcess.getId());
                audit.setBusinessId(bid);
                audit.setBusinessNumber(businessNumber);
                audit.setType(auditProcess.getType());
                audit.setAuditStatus(0);
                User user = userService.getCurrentUser();
                audit.setSubmitUser(user.getId());
                audit.setSubmitTime(new Date());
                //添加节点实例
                auditMapper.insert(audit);
                AuditNodeConfig auditNodeConfig = auditNodeConfigMapper.selectOne(new LambdaQueryWrapperX<AuditNodeConfig>()
                        .eq(AuditNodeConfig::getProcessId, auditProcess.getId())
                        .eq(AuditNodeConfig::getNodeOrder, 1));
                AuditNode auditNode = buildAuditNode(audit.getId(), auditNodeConfig, user.getId());
                auditNodeMapper.insert(auditNode);
            } else {
                //无审核流程，直接通过
                approved(bid, type);
            }
        } catch (Exception e) {
            log.error("创建审核流程失败", e);
            throw new ServiceException(StringUtil.isEmpty(e.getMessage()) ? "创建审核流程失败" : e.getMessage());
        }
        return true;
    }

    /**
     * 待我审批列表
     */
    @Override
    public List<AuditVo> pendingApprovalList(AuditQueryDTO auditQueryDTO) {
        User user = userService.getCurrentUser();
        auditQueryDTO.setUid(user.getId());
        PageUtils.startPage();
        List<AuditVo> list = auditMapper.pendingApprovalList(auditQueryDTO);
        list.forEach(v -> v.setAuditorName(user.getUsername()) );
        return list;
    }

    /**
     * 待我审批数量
     */
    @Override
    public Long pendingApprovalCount() {
        User user = userService.getCurrentUser();
        return auditNodeMapper.selectCount(AuditNode::getAuditorUser, user.getId(), AuditNode::getAuditResult, 0);
    }

    /**
     * 已处理审批列表
     */
    @Override
    public List<AuditVo> processedApprovalList(AuditQueryDTO auditQueryDTO) {
        User user = userService.getCurrentUser();
        auditQueryDTO.setUid(user.getId());
        PageUtils.startPage();
        List<AuditVo> list = auditMapper.processedApprovalList(auditQueryDTO);
        list.forEach(v -> v.setAuditorName(user.getUsername()));
        return list;
    }

    /**
     * 我发起的任务列表
     */
    @Override
    public List<AuditVo> myApprovalList(AuditQueryDTO auditQueryDTO) {
        User user = userService.getCurrentUser();
        auditQueryDTO.setUid(user.getId());
        PageUtils.startPage();
        List<AuditVo> list = auditMapper.myApprovalList(auditQueryDTO);
        list.forEach(v -> v.setInitiatorName(user.getUsername()));
        return list;
    }

    /**
     * 根据审核id获取审核历史
     */
    @Override
    public List<AuditNodeVo> getAuditHistory(Long id) {
        return auditNodeMapper.getAuditHistoryByAuditId(id);
    }

    /**
     * 设置审核状态
     *
     * @param auditDTO 审核参数
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public boolean setStatus(AuditDTO auditDTO) {
        User user = userService.getCurrentUser();
        //获取当前审核节点
        AuditNode auditNode = auditNodeMapper.selectOne(new LambdaQueryWrapperX<AuditNode>()
                .eq(AuditNode::getAuditId, auditDTO.getId())
                .eq(AuditNode::getAuditResult, 0)
                .eq(AuditNode::getDeleteFlag, false));
        //修改当前节点状态
        auditNode.setAuditorUser(user.getId());
        auditNode.setAuditResult(auditDTO.getStatus());
        auditNode.setAuditTime(new Date());
        auditNode.setAuditComment(auditDTO.getAuditComment());
        auditNodeMapper.updateById(auditNode);
        if (auditDTO.getStatus() == 1) {  //审批通过
            //获取下一个节点配置，判定是否有下一节点审核
            AuditNodeConfig auditNodeConfig = auditNodeConfigMapper.getAuditNodeConfigByAuditIdAndOrder(auditDTO.getId(), auditNode.getNodeOrder());
            if (auditNodeConfig != null && auditNodeConfig.getAuditorType() != null) {
                //有，根据节点配置生成下一节点
                //发起审核用户ID
                Long uid = getById(auditDTO.getId()).getSubmitUser();
                AuditNode newAuditNode = buildAuditNode(auditDTO.getId(), auditNodeConfig, uid);
                auditNodeMapper.insert(newAuditNode);
            } else {
                //无下一节点,审核阶段结束,修改对应审核内容状态
                //修改审核状态
                this.update(new UpdateWrapper<Audit>().set("audit_status", 1).eq("id", auditDTO.getId()));
                Audit audit = auditMapper.selectById(auditDTO.getId());
                approved(audit.getBusinessId(), audit.getType());
            }
        } else {
            //审批未通过
            //修改审核状态
            this.update(new UpdateWrapper<Audit>().set("audit_status", 2).eq("id", auditDTO.getId()));
            //修改单据状态
            Audit audit = auditMapper.selectById(auditDTO.getId());
            Long businessId = audit.getBusinessId();
            if (audit.getType().equals("盘点任务")) {
                taskStocktakingService.update(new UpdateWrapper<TaskStocktaking>().set("task_status", 8).eq("id", businessId));
            } else if (audit.getType().equals("pda维护商品")) {
                materialInputService.update(new UpdateWrapper<MaterialInput>().set("status", 8).eq("id", businessId));
            } else {
                depotHeadService.update(new UpdateWrapper<DepotHead>().set("status", 8).eq("id", businessId));
            }
        }
        return true;
    }

    /**
     * pda复核任务列表
     *
     * @param pdaDepotHeadDTO  筛选条件
     */
    @Override
    public List<PDADepotHeadVO> pdaList(PDADepotHeadDTO pdaDepotHeadDTO) {
        return auditMapper.pdaList(pdaDepotHeadDTO);
    }

    /**
     * 根据节点配置构建审核节点实例
     *
     * @param aid             审核id
     * @param auditNodeConfig 审核节点配置
     * @param uid            发起用户id
     */
    private AuditNode buildAuditNode(Long aid, AuditNodeConfig auditNodeConfig, Long uid) {
        AuditNode auditNode = new AuditNode();
        auditNode.setAuditId(aid);
        auditNode.setNodeId(auditNodeConfig.getId());
        auditNode.setNodeOrder(auditNodeConfig.getNodeOrder());
        if (auditNodeConfig.getAuditor() == null) {
            if (auditNodeConfig.getAuditorType().equals(2)) {
                //上级部门最高领导审核
                Long id = userService.getAuditUserIds(uid, true);
                if (id == null) {
                    throw new BusinessRunTimeException(ExceptionConstants.AUDIT_APPROVAL_HIGHER_DEPARTMENTS_LEADER_IS_NOT_EXIST_CODE,
                            ExceptionConstants.AUDIT_APPROVAL_HIGHER_DEPARTMENTS_LEADER_IS_NOT_EXIST_MSG);
                }
                auditNode.setAuditorUser(id);
            } else if (auditNodeConfig.getAuditorType().equals(3)) {
                //本部门最高领导审核
                Long id = userService.getAuditUserIds(uid, false);
                if (id == null) {
                    throw new BusinessRunTimeException(ExceptionConstants.AUDIT_APPROVAL_DEPARTMENT_LEADER_IS_NOT_EXIST_CODE,
                            ExceptionConstants.AUDIT_APPROVAL_DEPARTMENT_LEADER_IS_NOT_EXIST_MSG);
                }
                auditNode.setAuditorUser(id);
            }
        } else {
            auditNode.setAuditorUser(auditNodeConfig.getAuditor());
        }
        auditNode.setCreateTime(new Date());
        return auditNode;
    }

    /**
     * 审批通过,处理对应业务
     *
     * @param businessId 业务id
     * @param type       业务类型
     */
    private void approved(Long businessId, String type) {
        if (type.equals("盘点任务")) {
            taskStocktakingService.update(new UpdateWrapper<TaskStocktaking>().set("task_status", 1).eq("id", businessId));
        } else if (type.equals("pda维护商品")) {
            materialInputService.update(new UpdateWrapper<MaterialInput>().set("status", 1).eq("id", businessId));
            materialInputApproved(businessId);
        } else {
            //单据处理
            depotHeadService.update(new UpdateWrapper<DepotHead>().set("status", 1).eq("id", businessId));
            materialBatchService.generateMaterialBatchByDepotHeadId(businessId);
        }
    }

    /**
     * 单据审核通过 处理商品录入信息
     *
     * @param id 单据主表id
     */
    private void materialInputApproved(Long id) {
        MaterialInput materialInput = materialInputService.getById(id);
        materialInputService.update(new UpdateWrapper<MaterialInput>().set("status", 1).eq("id", id));
        //商品信息
        MaterialDTO materialDTO = new MaterialDTO();
        materialDTO.setName(materialInput.getName());
        materialDTO.setImgName(materialInput.getImgName());
        materialDTO.setCategoryId(materialInput.getCategoryId());
        materialDTO.setDefaultPurchaseDecimal(materialInput.getDefaultPurchaseDecimal());
        materialDTO.setDefaultWholesaleDecimal(materialInput.getDefaultWholesaleDecimal());
        //商品拓展信息
        MaterialExtendDTO materialExtend = new MaterialExtendDTO();
        materialExtend.setStandard(materialInput.getStandard());
        materialExtend.setCommodityUnit(materialInput.getCommodityUnit());
        //商品upc
        MaterialUpc materialUpc = new MaterialUpc();
        materialUpc.setUpc(materialInput.getBarCode());
        //商品单位
        MaterialUnit materialUnit = new MaterialUnit();
        materialUnit.setName(materialInput.getCommodityUnit());
        materialUnit.setRatio(1);
        List<MaterialUnit> unitList = new ArrayList<>();
        unitList.add(materialUnit);
        materialExtend.setUnitList(unitList);
        List<MaterialUpc> upcList = new ArrayList<>();
        upcList.add(materialUpc);
        materialExtend.setUpcList(upcList);
        List<MaterialExtendDTO> materialExtendDTOList = new ArrayList<>();
        materialExtendDTOList.add(materialExtend);
        materialDTO.setMeList(materialExtendDTOList);
        //商品初始库存
        MaterialInitialStock materialInitialStock = new MaterialInitialStock();
        materialInitialStock.setId(materialInput.getDepotId());
        materialInitialStock.setPosition(materialInput.getPosition());
        List<MaterialInitialStock> initialStockList = new ArrayList<>();
        initialStockList.add(materialInitialStock);
        materialDTO.setStock(initialStockList);
        materialService.insertMaterial(materialDTO, null);
    }

}
